@@ -1,7 +1,11 @@
+#define QT_VERSION_MAJOR    5
+#define QT_VERSION_MINOR    7
+#define QT_VERSION_PATCH    0
+#define QT_VERSION_STR      "5.7.0"
+
 /* Everything */
 /* Compile time features */
 #define QT_LARGEFILE_SUPPORT 64
-#define QT_POINTER_SIZE 8
 
 // Compiler sub-arch support
 
@@ -13,48 +17,18 @@
 # define QT_LINKED_OPENSSL
 #endif
 
-#if defined(QT_NO_ALSA) && defined(QT_ALSA)
-# undef QT_NO_ALSA
-#elif !defined(QT_NO_ALSA) && !defined(QT_ALSA)
-# define QT_NO_ALSA
-#endif
-
 #if defined(QT_NO_IMAGEFORMAT_JPEG) && defined(QT_IMAGEFORMAT_JPEG)
 # undef QT_NO_IMAGEFORMAT_JPEG
 #elif !defined(QT_NO_IMAGEFORMAT_JPEG) && !defined(QT_IMAGEFORMAT_JPEG)
 # define QT_NO_IMAGEFORMAT_JPEG
 #endif
 
-#if defined(QT_NO_LIBPROXY) && defined(QT_LIBPROXY)
-# undef QT_NO_LIBPROXY
-#elif !defined(QT_NO_LIBPROXY) && !defined(QT_LIBPROXY)
-# define QT_NO_LIBPROXY
-#endif
-
 #if defined(QT_NO_OPENVG) && defined(QT_OPENVG)
 # undef QT_NO_OPENVG
 #elif !defined(QT_NO_OPENVG) && !defined(QT_OPENVG)
 # define QT_NO_OPENVG
 #endif
 
-#if defined(QT_NO_TABLET) && defined(QT_TABLET)
-# undef QT_NO_TABLET
-#elif !defined(QT_NO_TABLET) && !defined(QT_TABLET)
-# define QT_NO_TABLET
-#endif
-
-#if defined(QT_NO_TSLIB) && defined(QT_TSLIB)
-# undef QT_NO_TSLIB
-#elif !defined(QT_NO_TSLIB) && !defined(QT_TSLIB)
-# define QT_NO_TSLIB
-#endif
-
-#if defined(QT_NO_XINPUT) && defined(QT_XINPUT)
-# undef QT_NO_XINPUT
-#elif !defined(QT_NO_XINPUT) && !defined(QT_XINPUT)
-# define QT_NO_XINPUT
-#endif
-
 #if defined(QT_NO_ZLIB) && defined(QT_ZLIB)
 # undef QT_NO_ZLIB
 #elif !defined(QT_NO_ZLIB) && !defined(QT_ZLIB)
@@ -91,18 +65,18 @@
 # define QT_RUNTIME_XFIXES
 #endif
 
-#if defined(QT_RUNTIME_XINERAMA) && defined(QT_NO_RUNTIME_XINERAMA)
-# undef QT_RUNTIME_XINERAMA
-#elif !defined(QT_RUNTIME_XINERAMA) && !defined(QT_NO_RUNTIME_XINERAMA)
-# define QT_RUNTIME_XINERAMA
-#endif
-
 #if defined(QT_RUNTIME_XRANDR) && defined(QT_NO_RUNTIME_XRANDR)
 # undef QT_RUNTIME_XRANDR
 #elif !defined(QT_RUNTIME_XRANDR) && !defined(QT_NO_RUNTIME_XRANDR)
 # define QT_RUNTIME_XRANDR
 #endif
 
+#if defined(QT_THREADSAFE_CLOEXEC) && defined(QT_NO_THREADSAFE_CLOEXEC)
+# undef QT_THREADSAFE_CLOEXEC
+#elif !defined(QT_THREADSAFE_CLOEXEC) && !defined(QT_NO_THREADSAFE_CLOEXEC)
+# define QT_THREADSAFE_CLOEXEC 1
+#endif
+
 #endif // QT_BOOTSTRAPPED
 
 #define QT_VISIBILITY_AVAILABLE
